@@ -51,8 +51,6 @@ using namespace std;
 
 exec_context INIT_EXEC_CONTEXT;
 
-bookmark_type_t BM_QUERY("query");
-
 static const string MSG_FORMAT_STMT = R"(
 SELECT count(*) AS total, min(log_line) AS log_line, log_msg_format
     FROM all_logs
@@ -332,6 +330,10 @@ Result<string, string> execute_sql(exec_context &ec, const string &sql, string &
 
     gettimeofday(&end_tv, nullptr);
     if (retcode == SQLITE_DONE) {
+        if (lnav_data.ld_log_source.is_line_meta_changed()) {
+            lnav_data.ld_log_source.text_filters_changed();
+            lnav_data.ld_views[LNV_LOG].reload_data();
+        }
         lnav_data.ld_filter_view.reload_data();
         lnav_data.ld_files_view.reload_data();
         lnav_data.ld_views[LNV_DB].reload_data();
@@ -341,8 +343,6 @@ Result<string, string> execute_sql(exec_context &ec, const string &sql, string &
             retval = ec.ec_accumulator.get_string();
         }
         else if (!dls.dls_rows.empty()) {
-            vis_bookmarks &bm = lnav_data.ld_views[LNV_LOG].get_bookmarks();
-
             if (lnav_data.ld_flags & LNF_HEADLESS) {
                 if (ec.ec_local_vars.size() == 1) {
                     ensure_view(&lnav_data.ld_views[LNV_DB]);
@@ -351,13 +351,6 @@ Result<string, string> execute_sql(exec_context &ec, const string &sql, string &
                 retval = "";
                 alt_msg = "";
             }
-            else if (dls.dls_headers.size() == 1 && !bm[&BM_QUERY].empty()) {
-                retval = "";
-                alt_msg = HELP_MSG_2(
-                  y, Y,
-                  "to move forward/backward through query results "
-                  "in the log view");
-            }
             else if (dls.dls_rows.size() == 1) {
                 auto &row = dls.dls_rows[0];
 
@@ -625,7 +618,7 @@ Result<string, string> execute_any(exec_context &ec, const string &cmdline_with_
             (lnav_data.ld_flags & LNF_HEADLESS ||
              ec.ec_path_stack.size() > 1)) {
             rescan_files();
-            rebuild_indexes();
+            rebuild_indexes_repeatedly();
         }
     });
 
@@ -689,7 +682,7 @@ void execute_init_commands(exec_context &ec, vector<pair<Result<string, string>,
         }
 
         rescan_files();
-        rebuild_indexes();
+        rebuild_indexes_repeatedly();
 
         ec.ec_source.pop();
     }
@@ -723,7 +716,6 @@ int sql_callback(exec_context &ec, sqlite3_stmt *stmt)
 
     if (!sqlite3_stmt_busy(stmt)) {
         dls.clear();
-        lnav_data.ld_log_source.text_clear_marks(&BM_QUERY);
 
         return 0;
     }
@@ -770,16 +762,6 @@ int sql_callback(exec_context &ec, sqlite3_stmt *stmt)
                     break;
             }
         }
-        if (value != nullptr &&
-            (dls.dls_headers[lpc].hm_name == "log_line" ||
-             strstr(dls.dls_headers[lpc].hm_name.c_str(), "log_line"))) {
-            int line_number = -1;
-
-            if (sscanf(value, "%d", &line_number) == 1) {
-                lnav_data.ld_views[LNV_LOG].toggle_user_mark(
-                    &BM_QUERY, vis_line_t(line_number));
-            }
-        }
     }
 
     return retval;
@@ -866,23 +848,23 @@ std::string exec_context::get_error_prefix()
     return fmt::format("{}:{}: error: ", source.first, source.second);
 }
 
-void exec_context::set_output(const string &name, FILE *file)
+void exec_context::set_output(const string &name, FILE *file, int (*closer)(FILE *))
 {
     log_info("redirecting command output to: %s", name.c_str());
-    this->ec_output_stack.back().second | [](auto file) {
-        if (file != stdout) {
-            fclose(file);
+    this->ec_output_stack.back().second | [](auto out) {
+        if (out.second != nullptr) {
+            out.second(out.first);
         }
     };
-    this->ec_output_stack.back() = std::make_pair(name, file);
+    this->ec_output_stack.back() = std::make_pair(name, std::make_pair(file, closer));
 }
 
 void exec_context::clear_output()
 {
     log_info("redirecting command output to screen");
-    this->ec_output_stack.back().second | [](auto file) {
-        if (file != stdout) {
-            fclose(file);
+    this->ec_output_stack.back().second | [](auto out) {
+        if (out.second != nullptr) {
+            out.second(out.first);
         }
     };
     this->ec_output_stack.back() = std::make_pair("default", nonstd::nullopt);
@@ -890,7 +872,7 @@ void exec_context::clear_output()
 
 exec_context::output_guard::output_guard(exec_context &context,
                                          std::string name,
-                                         const nonstd::optional<FILE *> &file)
+                                         const nonstd::optional<output_t> &file)
     : sg_context(context) {
     if (file) {
         log_info("redirecting command output to: %s", name.c_str());
