@@ -43,6 +43,7 @@
 #include "vtab_module.hh"
 #include "vtab_module_json.hh"
 
+#include "lnav_util.hh"
 #include "yajl/api/yajl_gen.h"
 #include "sqlite-extension-func.hh"
 
@@ -53,11 +54,12 @@ using namespace mapbox;
 
 class sql_json_op : public json_op {
 public:
-    sql_json_op(json_ptr &ptr) : json_op(ptr), sjo_type(-1), sjo_int(0) { };
+    sql_json_op(json_ptr &ptr) : json_op(ptr) { };
 
-    int sjo_type;
+    int sjo_type{-1};
     string sjo_str;
-    int sjo_int;
+    int64_t sjo_int{0};
+    double sjo_float{0.0};
 };
 
 static void null_or_default(sqlite3_context *context, int argc, sqlite3_value **argv)
@@ -72,6 +74,7 @@ static void null_or_default(sqlite3_context *context, int argc, sqlite3_value **
 
 struct contains_userdata {
     util::variant<const char *, sqlite3_int64, bool> cu_match_value{false};
+    size_t cu_depth{0};
     bool cu_result{false};
 };
 
@@ -79,7 +82,8 @@ static int contains_string(void *ctx, const unsigned char *str, size_t len)
 {
     auto &cu = *((contains_userdata *) ctx);
 
-    if (strncmp((const char *) str, cu.cu_match_value.get<const char *>(), len) == 0) {
+    if (cu.cu_depth <= 1 &&
+        strncmp((const char *) str, cu.cu_match_value.get<const char *>(), len) == 0) {
         cu.cu_result = true;
     }
 
@@ -90,15 +94,29 @@ static int contains_integer(void *ctx, long long value)
 {
     auto &cu = *((contains_userdata *) ctx);
 
-    if (cu.cu_match_value.get<sqlite3_int64>() == value) {
+    if (cu.cu_depth <= 1 && cu.cu_match_value.get<sqlite3_int64>() == value) {
         cu.cu_result = true;
     }
 
     return 1;
 }
 
-static bool json_contains(const char *json_in, sqlite3_value *value)
+static int contains_null(void *ctx)
 {
+    auto &cu = *((contains_userdata *) ctx);
+
+    cu.cu_result = true;
+
+    return 1;
+}
+
+static bool json_contains(vtab_types::nullable<const char> nullable_json_in, sqlite3_value *value)
+{
+    if (nullable_json_in.n_value == nullptr || nullable_json_in.n_value[0] == '\0') {
+        return false;
+    }
+
+    auto json_in = nullable_json_in.n_value;
     auto_mem<yajl_handle_t> handle(yajl_free);
     yajl_callbacks cb;
     contains_userdata cu;
@@ -106,6 +124,35 @@ static bool json_contains(const char *json_in, sqlite3_value *value)
     memset(&cb, 0, sizeof(cb));
     handle = yajl_alloc(&cb, nullptr, &cu);
 
+    cb.yajl_start_array = +[](void *ctx) {
+        auto &cu = *((contains_userdata *) ctx);
+
+        cu.cu_depth += 1;
+
+        return 1;
+    };
+    cb.yajl_end_array = +[](void *ctx) {
+        auto &cu = *((contains_userdata *) ctx);
+
+        cu.cu_depth -= 1;
+
+        return 1;
+    };
+    cb.yajl_start_map = +[](void *ctx) {
+        auto &cu = *((contains_userdata *) ctx);
+
+        cu.cu_depth += 2;
+
+        return 1;
+    };
+    cb.yajl_end_map = +[](void *ctx) {
+        auto &cu = *((contains_userdata *) ctx);
+
+        cu.cu_depth -= 2;
+
+        return 1;
+    };
+
     switch (sqlite3_value_type(value)) {
         case SQLITE3_TEXT:
             cb.yajl_string = contains_string;
@@ -115,6 +162,9 @@ static bool json_contains(const char *json_in, sqlite3_value *value)
             cb.yajl_integer = contains_integer;
             cu.cu_match_value = sqlite3_value_int64(value);
             break;
+        case SQLITE_NULL:
+            cb.yajl_null = contains_null;
+            break;
     }
 
     if (yajl_parse(handle.in(), (const unsigned char *) json_in, strlen(json_in)) != yajl_status_ok ||
@@ -172,6 +222,28 @@ static int gen_handle_string(void *ctx, const unsigned char * stringVal, size_t
     return sjo->jo_ptr_error_code == yajl_gen_status_ok;
 }
 
+static int gen_handle_number(void *ctx, const char *numval, size_t numlen)
+{
+    sql_json_op *sjo = (sql_json_op *)ctx;
+    yajl_gen gen = (yajl_gen)sjo->jo_ptr_data;
+
+    if (sjo->jo_ptr.jp_state == json_ptr::MS_DONE) {
+        if (strtonum(sjo->sjo_int, numval, numlen) == numlen) {
+            sjo->sjo_type = SQLITE_INTEGER;
+        } else {
+            auto numstr = std::string(numval, numlen);
+
+            sjo->sjo_float = std::stod(numstr);
+            sjo->sjo_type = SQLITE_FLOAT;
+        }
+    }
+    else {
+        sjo->jo_ptr_error_code = yajl_gen_number(gen, numval, numlen);
+    }
+
+    return sjo->jo_ptr_error_code == yajl_gen_status_ok;
+}
+
 static void sql_jget(sqlite3_context *context,
                      int argc, sqlite3_value **argv)
 {
@@ -205,6 +277,7 @@ static void sql_jget(sqlite3_context *context,
     jo.jo_ptr_callbacks.yajl_null = gen_handle_null;
     jo.jo_ptr_callbacks.yajl_boolean = gen_handle_boolean;
     jo.jo_ptr_callbacks.yajl_string = gen_handle_string;
+    jo.jo_ptr_callbacks.yajl_number = gen_handle_number;
     jo.jo_ptr_data = gen.get_handle();
 
     handle.reset(yajl_alloc(&json_op::ptr_callbacks, nullptr, &jo));
@@ -258,6 +331,9 @@ static void sql_jget(sqlite3_context *context,
     case SQLITE_INTEGER:
         sqlite3_result_int(context, jo.sjo_int);
         return;
+    case SQLITE_FLOAT:
+        sqlite3_result_double(context, jo.sjo_float);
+        return;
     }
 
     string_fragment result = gen.to_string_fragment();
@@ -268,6 +344,9 @@ static void sql_jget(sqlite3_context *context,
     }
 
     sqlite3_result_text(context, result.data(), result.length(), SQLITE_TRANSIENT);
+#ifdef HAVE_SQLITE3_VALUE_SUBTYPE
+    sqlite3_result_subtype(context, JSON_SUBTYPE);
+#endif
 }
 
 struct concat_context {
