@@ -33,6 +33,7 @@
 #include "base/string_util.hh"
 #include "sql_util.hh"
 #include "log_vtab_impl.hh"
+#include "yajlpp/json_op.hh"
 #include "yajlpp/yajlpp_def.hh"
 #include "vtab_module.hh"
 
@@ -40,6 +41,8 @@
 
 using namespace std;
 
+static auto intern_lifetime = intern_string::get_table_lifetime();
+
 static struct log_cursor log_cursor_latest;
 
 struct _log_vtab_data log_vtab_data;
@@ -622,6 +625,27 @@ static int vt_column(sqlite3_vtab_cursor *cur, sqlite3_context *ctx, int col)
                                 case value_kind_t::VALUE_FLOAT:
                                     root.gen(lv_struct.lv_value.d);
                                     break;
+                                case value_kind_t::VALUE_JSON: {
+                                    auto_mem<yajl_handle_t> parse_handle(yajl_free);
+                                    json_ptr jp("");
+                                    json_op jo(jp);
+
+                                    jo.jo_ptr_callbacks = json_op::gen_callbacks;
+                                    jo.jo_ptr_data = gen;
+                                    parse_handle.reset(yajl_alloc(&json_op::ptr_callbacks, nullptr, &jo));
+
+                                    auto json_in = (const unsigned char *) lv_struct.text_value();
+                                    auto json_len = lv_struct.text_length();
+
+                                    if (yajl_parse(parse_handle.in(), json_in, json_len) != yajl_status_ok ||
+                                        yajl_complete_parse(parse_handle.in()) != yajl_status_ok) {
+                                        log_error("failed to parse json value: %.*s",
+                                                  lv_struct.text_length(),
+                                                  lv_struct.text_value());
+                                        root.gen(lv_struct.to_string());
+                                    }
+                                    break;
+                                }
                                 default:
                                     root.gen(lv_struct.to_string());
                                     break;
@@ -792,14 +816,14 @@ static int vt_filter(sqlite3_vtab_cursor *p_vtc,
                 date_time_scanner dts;
                 struct timeval tv;
                 struct exttm mytm;
-                vis_line_t vl;
 
                 dts.scan((const char *)datestr, strlen((const char *)datestr), NULL, &mytm, tv);
-                if ((vl = vt->lss->find_from_time(tv)) == -1) {
+                auto vl_opt = vt->lss->find_from_time(tv);
+                if (!vl_opt) {
                     p_cur->log_cursor.lc_curr_line = p_cur->log_cursor.lc_end_line;
                 }
                 else {
-                    p_cur->log_cursor.update(index[lpc].op, vl, false);
+                    p_cur->log_cursor.update(index[lpc].op, vl_opt.value(), false);
                 }
             }
             break;
@@ -938,6 +962,7 @@ static int vt_update(sqlite3_vtab *tab,
         if (binary_search(bv.begin(), bv.end(), vrowid) && !has_meta) {
             vt->tc->set_user_mark(&textview_curses::BM_META, vrowid, false);
             bm.erase(vt->lss->at(vrowid));
+            vt->lss->set_line_meta_changed();
         }
 
         if (has_meta) {
@@ -966,6 +991,8 @@ static int vt_update(sqlite3_vtab *tab,
             } else {
                 line_meta.bm_tags.clear();
             }
+
+            vt->lss->set_line_meta_changed();
         }
 
         vt->tc->set_user_mark(&textview_curses::BM_USER, vrowid, val);
@@ -1073,10 +1100,10 @@ string log_vtab_manager::register_vtab(std::shared_ptr<log_vtab_impl> vi)
 
 string log_vtab_manager::unregister_vtab(intern_string_t name)
 {
-    string retval = "";
+    string retval;
 
     if (this->vm_impls.find(name) == this->vm_impls.end()) {
-        retval = "unknown log line table -- " + name.to_string();
+        retval = fmt::format("unknown log line table -- {}", name);
     }
     else {
         auto_mem<char, sqlite3_free> sql;
