@@ -53,6 +53,8 @@
 
 using namespace std;
 
+static auto intern_lifetime = intern_string::get_table_lifetime();
+
 static const size_t INDEX_RESERVE_INCREMENT = 1024;
 
 Result<std::shared_ptr<logfile>, std::string> logfile::open(
@@ -330,11 +332,15 @@ logfile::rebuild_result_t logfile::rebuild_index(nonstd::optional<ui_clock::time
         return rebuild_result_t::INVALID;
     }
 
+    const auto is_truncated = st.st_size < this->lf_stat.st_size;
+    const auto is_user_provided_and_rewritten = (
+        // files from other sources can have their mtimes monkeyed with
+        this->lf_options.loo_source == logfile_name_source::USER &&
+        this->lf_stat.st_size == st.st_size &&
+        this->lf_stat.st_mtime != st.st_mtime);
+
     // Check the previous stat against the last to see if things are wonky.
-    if (this->lf_named_file &&
-        (st.st_size < this->lf_stat.st_size ||
-         (this->lf_stat.st_size == st.st_size &&
-          this->lf_stat.st_mtime != st.st_mtime))) {
+    if (this->lf_named_file && (is_truncated || is_user_provided_and_rewritten)) {
         log_info("overwritten file detected, closing -- %s  new: %" PRId64
                      "/%" PRId64 "  old: %" PRId64 "/%" PRId64,
                  this->lf_filename.c_str(),
@@ -407,7 +413,7 @@ logfile::rebuild_result_t logfile::rebuild_index(nonstd::optional<ui_clock::time
 
         bool sort_needed = this->lf_sort_needed;
         this->lf_sort_needed = false;
-        size_t limit = 1000 * 1000;
+        size_t limit = SIZE_MAX;
 
         if (deadline) {
             if (ui_clock::now() > deadline.value()) {
@@ -420,6 +426,8 @@ logfile::rebuild_result_t logfile::rebuild_index(nonstd::optional<ui_clock::time
                 }
             } else if (!has_format) {
                 limit = 1000;
+            } else {
+                limit = 1000 * 1000;
             }
         }
         if (!has_format) {
