@@ -31,8 +31,6 @@
 
 #include "config.h"
 #include <string.h>
-#include <langinfo.h>
-#include <set>
 #include <algorithm>
 
 #include "ptimec.hh"
@@ -67,11 +65,15 @@ bool ptime_fmt(const char *fmt, struct exttm *dst, const char *str, off_t &off,
                 case 'a':
                 case 'Z':
                     if (fmt[lpc + 2]) {
-                        if (!ptime_upto(fmt[lpc + 2], str, off, len)) return false;
+                        if (!ptime_upto(fmt[lpc + 2], str, off, len)) {
+                            return false;
+                        }
                         lpc += 1;
                     }
                     else {
-                        if (!ptime_upto_end(str, off, len)) return false;
+                        if (!ptime_upto_end(str, off, len)) {
+                            return false;
+                        }
                         lpc += 1;
                     }
                     break;
@@ -82,6 +84,7 @@ bool ptime_fmt(const char *fmt, struct exttm *dst, const char *str, off_t &off,
                 FMT_CASE('M', M);
                 FMT_CASE('H', H);
                 FMT_CASE('i', i);
+                FMT_CASE('6', 6);
                 FMT_CASE('I', I);
                 FMT_CASE('d', d);
                 FMT_CASE('e', e);
@@ -91,6 +94,7 @@ bool ptime_fmt(const char *fmt, struct exttm *dst, const char *str, off_t &off,
                 FMT_CASE('m', m);
                 FMT_CASE('N', N);
                 FMT_CASE('p', p);
+                FMT_CASE('q', q);
                 FMT_CASE('Y', Y);
                 FMT_CASE('y', y);
                 FMT_CASE('z', z);
@@ -129,6 +133,7 @@ size_t ftime_fmt(char *dst, size_t len, const char *fmt, const struct exttm &tm)
                 FTIME_FMT_CASE('M', M);
                 FTIME_FMT_CASE('H', H);
                 FTIME_FMT_CASE('i', i);
+                FTIME_FMT_CASE('6', 6);
                 FTIME_FMT_CASE('I', I);
                 FTIME_FMT_CASE('d', d);
                 FTIME_FMT_CASE('e', e);
@@ -138,6 +143,7 @@ size_t ftime_fmt(char *dst, size_t len, const char *fmt, const struct exttm &tm)
                 FTIME_FMT_CASE('m', m);
                 FTIME_FMT_CASE('N', N);
                 FTIME_FMT_CASE('p', p);
+                FTIME_FMT_CASE('q', q);
                 FTIME_FMT_CASE('Y', Y);
                 FTIME_FMT_CASE('y', y);
                 FTIME_FMT_CASE('z', z);
