@@ -59,9 +59,14 @@ static nonstd::optional<std::string> sql_log_top_datetime()
         return nonstd::nullopt;
     }
 
+    auto top_time = lnav_data.ld_log_source.time_for_row(lnav_data.ld_views[LNV_LOG].get_top());
+    if (!top_time) {
+        return nonstd::nullopt;
+    }
+
     char buffer[64];
 
-    sql_strftime(buffer, sizeof(buffer), lnav_data.ld_log_source.time_for_row(lnav_data.ld_views[LNV_LOG].get_top()));
+    sql_strftime(buffer, sizeof(buffer), top_time.value());
     return buffer;
 }
 
@@ -83,6 +88,11 @@ static nonstd::optional<std::string> sql_lnav_top_file()
     });
 }
 
+static const char *sql_lnav_version()
+{
+    return PACKAGE_VERSION;
+}
+
 static int64_t sql_error(const char *str)
 {
     throw sqlite_func_error("{}", str);
@@ -110,6 +120,12 @@ int state_extension_functions(struct FuncDef **basic_funcs,
                 .sql_function()
         ),
 
+        sqlite_func_adapter<decltype(&sql_lnav_version), sql_lnav_version>::builder(
+            help_text("lnav_version",
+                      "Return the current version of lnav")
+                .sql_function()
+        ),
+
         sqlite_func_adapter<decltype(&sql_error), sql_error>::builder(
             help_text("raise_error",
                       "Raises an error with the given message when executed")
